package com.telpo.iotgateway.listener;

import com.google.common.base.Stopwatch;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.jms.Message;
import javax.jms.MessageListener;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @program: iotgateway
 * @description: Iot消息侦听
 * @author: linwl
 * @create: 2021-02-24 11:39
 **/
@Slf4j
@Component
public class IotMessageListener implements MessageListener {

    private AtomicInteger count = new AtomicInteger(0);
    /**
     * Runnable：实现了Runnable接口，jdk就知道这个类是一个线程
     */
    private Runnable runnable;

    public IotMessageListener() {
        this.runnable = new Runnable() {
            //创建 run 方法
            @Override
            public void run() {
                // task to run goes here
                count.getAndSet(0);
                System.out.println("Hello, stranger");
            }
        };

        // ScheduledExecutorService:是从Java SE5的java.util.concurrent里，
        // 做为并发工具类被引进的，这是最理想的定时任务实现方式。
        ScheduledExecutorService service = Executors.newSingleThreadScheduledExecutor();
        // 第二个参数为首次执行的延时时间，第三个参数为定时执行的间隔时间
        // 10：秒   5：秒
        // 第一次执行的时间为10秒，然后每隔五秒执行一次
        service.scheduleAtFixedRate(runnable, 60, 60, TimeUnit.SECONDS);
    }

    private ExecutorService executorService;
    public void setExecutorService(ExecutorService executorService) {
        this.executorService = executorService;
    }

    @Override
    public void onMessage(Message message) {
        try {
            //1.收到消息之后一定要ACK。
            // 推荐做法：创建Session选择Session.AUTO_ACKNOWLEDGE，这里会自动ACK。
            // 其他做法：创建Session选择Session.CLIENT_ACKNOWLEDGE，这里一定要调message.acknowledge()来ACK。
            // message.acknowledge();
            //2.异步触发回调，处理收到的消息，确保onMessage函数里没有耗时逻辑。
            // 如果业务处理耗时过程过长阻塞住线程，可能会影响SDK收到消息后的正常回调。
            executorService.submit(() -> processMessage(message));
        } catch (Exception e) {
            log.error("submit task occurs exception ", e);
        }
    }

    /**
     * 在这里处理您收到消息后的具体业务逻辑。
     */
    private void processMessage(Message message) {
        try {
            byte[] body = message.getBody(byte[].class);
            String content = new String(body);
            String topic = message.getStringProperty("topic");
            String messageId = message.getStringProperty("messageId");
            log.info("receive message"
                    + ", topic = " + topic
                    + ", messageId = " + messageId
                    + ", content = " + content);
            count.getAndIncrement();

        } catch (Exception e) {
            log.error("processMessage occurs error ", e);
        }
    }



//    @Scheduled(cron = "${scheduler.task.cron}")
//    public void countMessages() {
//        //log.info("开始执行定时推送失败的推送记录!");
//        // 获取推送失败的记录
//        //try {
//        log.warn("约1分钟处理 {} 个请求",count);
//        count.getAndSet(0);
//        //} catch (Exception e) {
//        //    log.error("执行定时计数发生异常:", e);
//        //}
//    }

}
